// LFindWindow.cpp

#include "LFindWindow.h"
#include "LMainWindow.h"
#include "listItemData.h"

struct LFindWindow_impl {
	LMainWindow* lmw;
	LFindWindow* lfw;

	// Controls
	MGEdit edEntry;
	MGListBox lbResults;
	MGButton btOK, btCancel;

	// Last search
	std::string lastSearch;

	// 
	LFindWindow_impl(LFindWindow* lfw, LMainWindow* lmw) :
		lfw(lfw), lmw(lmw),
		edEntry(lfw, false, false),
		lbResults(lfw, false, false, 1, false),
		btOK(lfw, "&OK"),
		btCancel(lfw, "&Cancel")
	{
	}

	void doResize() {
		int h = lfw->getInnerHeight(), w = lfw->getInnerWidth();

		edEntry.resize(5, 5, w - 10, 20);
		lbResults.resize(5, 30, w - 10, h - 65);
		btOK.resize(w - 150, h - 30, 70, 25);
		btCancel.resize(w - 75, h - 30, 70, 25);
	};
};

// s

LFindWindow::LFindWindow(MGWindow* parent) :
	MGWindow( (void*)IDI_LILLYMAIN, "Find a file in current list", parent),
	p(new LFindWindow_impl(this, static_cast<LMainWindow*>(parent))),
	finalSelection(NOTHING_SELECTED)
{
	this->setResizable(true);
	this->resize(480, 320);	
	p->doResize();
	this->setVisible(true);

	onResize.assign(this, MGCB(LFindWindow::doResize));
	p->edEntry.onKey.assign(this, MGCB(LFindWindow::onEditorKeypress));
	p->lbResults.onDoubleClick.assign(this, MGCB(LFindWindow::onEntryDoubleClicked));
	p->lbResults.onKey.assign(this, MGCB(LFindWindow::onEntryKeypress));

	p->btOK.onClick.assign(this, MGCB(LFindWindow::onOK));
	p->btCancel.onClick.assign(this, MGCB(LFindWindow::onCancel));

	p->edEntry.focus();

	//this->onKey.assign(this, MGCB(LFindWindow::onFormKeypress));

	// Keep a copy of all items in the main list, lowercase'd.
	// Since this dialog is modal, we can use the index to retrieve the original
	//   case on display
	lowNames.clear();
	MGListBox *lbMain = p->lmw->exposeListBox();
	
	lowNames.reserve(lbMain->getItemCount());
	wNames.reserve(lowNames.size());

	for (int i = 0, e = lbMain->getItemCount(); i < e; i++) {

		// Can't use getItemText, as this might contain queue indicators
		listItemData* lid = (listItemData*)lbMain->getItemData(i);
		std::string ret = lid->format( DISPLAY_NAMEONLY /* always? */);
	
		wNames.push_back(utf8to16(ret.c_str()));

		strtolower(ret);
		lowNames.push_back(ret);
	};
}
	
LFindWindow::~LFindWindow() { delete p; }
void LFindWindow::doResize() { p->doResize(); }

void LFindWindow::setSearchText(const char* text) {
	p->edEntry.textSet(text);
	p->edEntry.selEnd();
	doUpdateSearch();
}

void LFindWindow::onOK() {
	unsigned int tmp = p->lbResults.getSelected();
	if (tmp == NOTHING_SELECTED) {
		finalSelection = NOTHING_SELECTED;
	} else {
		finalSelection = (unsigned int) p->lbResults.getItemData(tmp); // hard cast
	}
	close();
};

void LFindWindow::onCancel() {
	finalSelection = NOTHING_SELECTED;
	close();
};

void LFindWindow::onEntryDoubleClicked() {
	onOK(); // calls close();
};

void LFindWindow::onEntryKeypress() {
	MGKeyCombination kc = p->lbResults.onKey.data;

	if (kc.virtkey == MGVK_ENTER) {
		onOK(); // calls close();

	} else if (kc.virtkey == MGVK_ESC) {
		close();

	} else if (
		!kc.alt && !kc.ctrl && kc.virtkey >= MGVK_0 && kc.virtkey <= MGVK_Z
	) {
		// redirect input to editor
		p->edEntry.focus();
		p->edEntry.textAppend( MGKC_FormatCharacter(kc).c_str() );
	}

};

void LFindWindow::onFormKeypress() {
	// Hit if the editor hasn't yet been focused
	MGVirtualKeys vk = this->onKey.data.virtkey;
	if (vk == MGVK_ESC) {
		close();
	}
}

void LFindWindow::onEditorKeypress() {
	MGVirtualKeys vk = p->edEntry.onKey.data.virtkey;
	if (vk == MGVK_ESC) {
		close();
	} else if (vk == MGVK_DOWN) {
		p->lbResults.focus();
		p->lbResults.setSelected(0);
	}
	doUpdateSearch();
}

// 

void LFindWindow::doUpdateSearch() {
	// If the current text is a strict addition to what's already there,
	// search the existing list - otherwise, do a full re-search
	std::string curSearch = p->edEntry.textGet();
	strtolower(curSearch);
	
	if (curSearch == p->lastSearch) return;

	if (! curSearch.size()) {
		p->lbResults.clear();

	} else if (p->lastSearch.size() && curSearch.find(p->lastSearch) == 0) {
		// Probably the overhead of all this multiple pointer indirection now means
		//  that this 'optimisation' is slower than just doing a full search on
		//  the existing lowNames vector...

		for (int i = 0, e = p->lbResults.getItemCount(); i < e ; i++) {
			std::string item = lowNames[ (unsigned int)p->lbResults.getItemData(i) ];
			//std::string item = p->lbResults.getItemText(i);
			if (item.find(curSearch) == item.npos) {
				p->lbResults.removeItem(i);
				i--;
				e--;
			}
		}

	} else {
		p->lbResults.clear();
		p->lbResults.setVisible(false);

		MGListBox *lbMain = p->lmw->exposeListBox();

		for (unsigned int i = 0, e = lowNames.size(); i < e; i++) {
			if (lowNames[i].find(curSearch) != std::string::npos) {
				unsigned int pos = p->lbResults.addItem_UTF16LE( wNames[i].c_str() ); //lbMain->getItemText(i).c_str() ); //lowNames[i].c_str());
				p->lbResults.setItemData( pos, (void*)i ); // hard cast
			}
		}
		p->lbResults.setVisible(true);
	}

	p->lastSearch = curSearch;
};
