// LMainWindow.cpp
// ```````````````

#include "stdafx.h"
#include "LMainWindow.h"
#include "osal.h"
#include "SettingsManager.h"

#define LMAINWINDOW_INTERNAL
#include "LMainWindow_ui.h"
#undef LMAINWINDOW_INTERNAL

#include "listItemData.h"

static const char* extensions[] = {
	".mp3", ".m4a", ".aac", ".ogg", ".ogm", ".flac", ".wav", ".wma"
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// CONSTRUCTION / DECONSTRUCTION
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

LMainWindow::LMainWindow() :
MGWindow((void*)IDI_LILLYMAIN, APPLICATION_TITLE, NULL)
{
	
	// Restore window position
	this->resize(
		LS_GET(LS_WindowXPos),  LS_GET(LS_WindowYPos),
		LS_GET(LS_WindowWidth), LS_GET(LS_WindowHeight)
	);

	// Window properties
	this->showStatusBar(true);
	this->onClosing.assign(this, MGCB(LMainWindow::OnWindowClosing));
	this->onResize.assign(this, MGCB(LMainWindow::OnWindowResize));
	this->onMove.assign(this, MGCB(LMainWindow::OnWindowMove));

	//this->onRepaint.assign(this, MGCB(LMainWindow::OnRepaint));

	// Child windows
	//wndSettings = 0;
	
	// Create child UI elements
	ui = new LMainWindowUI(this);

	// UI element events
	ui->pbProgress.onDrag.assign(this, MGCB(LMainWindow::OnProgressDragged));
	ui->pbProgress.onMouseDown.assign(this, MGCB(LMainWindow::OnProgressMDown));
	ui->lbMain.onDoubleClick.assign(this, MGCB(LMainWindow::OnListDoubleClick));
	ui->lbMain.setAllowTypeToSelect(false);

	onDropFiles.assign(this, MGCB(LMainWindow::OnDropFiles));
	refreshHasDropFilesHandler();

	// Register hotkeys
	hkNextSong = new MGHotkey(    this, LS_MGKC(Next) );
	hkNextSong->onPress.assign(   this, MGCB(LMainWindow::playNext) );
	hkPlayPause = new MGHotkey(   this, LS_MGKC(PlayPause));
	hkPlayPause->onPress.assign(  this, MGCB(LMainWindow::OnPlayPause) );
	hkFocusWindow = new MGHotkey( this, LS_MGKC(Focus));
	hkFocusWindow->onPress.assign(this, MGCB(LMainWindow::OnHotkeyFocusWindow) );
	
	this->updateMenu();
	ui->postMenuInitialisation();

	onKey.assign( this, MGCB( LMainWindow::OnKeypress_form ) );
	ui->lbMain.onKey.assign( this, MGCB( LMainWindow::OnKeypress_lb ) );

	// Audio initialisation

	hCurrent = 0;
	lBitrate = 0;
	setShuffleMode(MODE_SHUFFLE);
	setDisplayMode(DISPLAY_NAMEONLY); // Redraws all text, so call before first playlist load
	tmrBASS = new MGTimer(this, 20);
	tmrBASS->onTick.assign(this, MGCB(LMainWindow::OnBASSTimer));
	OnBASSTimer();
	
	// Showtime
	this->setVisible(true);
		
	// Load playlist from file
	this->loadPlaylist(std::string( APPLICATION_TITLE PLAYLIST_EXTENSION ));
}

#define LS_SET_MGKC(x, kc) { \
	MGKeyCombination _tmp = kc; \
	LS_SET( LS_Key ## x ## VK ,  (kc).virtkey ); \
	LS_SET( LS_Key ## x ## Alt,  (kc).alt ? 1 : 0); \
	LS_SET( LS_Key ## x ## Ctrl, (kc).ctrl ? 1 : 0); \
	LS_SET( LS_Key ## x ## Shift,(kc).shift ? 1 : 0); \
}
/*
void LMainWindow::OnRepaint() {
	//llVolume->setText("repaint");
	llProgress->setText(sCurrentProgress.c_str());
}*/

void LMainWindow::OnWindowClosing() {
	// Save last window position
	LS_SET(LS_WindowWidth,  this->getWidth());
	LS_SET(LS_WindowHeight, this->getHeight());
	LS_SET(LS_WindowXPos,   this->getX());
	LS_SET(LS_WindowYPos,   this->getY());

	LS_SET_MGKC(Next, this->hkNextSong->getCombination());
	LS_SET_MGKC(PlayPause, this->hkPlayPause->getCombination());
	LS_SET_MGKC(Focus, this->hkFocusWindow->getCombination());

	// Save playlist to file
	this->savePlaylist(std::string( APPLICATION_TITLE PLAYLIST_EXTENSION ));
}

LMainWindow::~LMainWindow() {
	
}

MGListBox* LMainWindow::exposeListBox() {
	return & ui->lbMain ;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// PUBLIC FUNCTIONS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void CALLBACK LMainWindow::EventProc(HSYNC handle, DWORD channel, DWORD data, void* user) {
	((LMainWindow*)user)->playNext();
}
/*
void LMainWindow::doChildWindowEvents() {
	if (wndSettings) wndSettings->doEvents();
}*/

void LMainWindow::clear() {

	// Stop - we don't play files that arn't in the list
	OnPlayStop();
	curListIndex = -1;

	// Free all item data memory before clearing the listbox
	for (unsigned int i = 0, e = ui->lbMain.getItemCount(); i != e; ++i) {
		delete ( (listItemData*)ui->lbMain.getItemData(i) );
	}

	// Clear queue
	qIndexes.clear();

	// Clear listbox
	ui->lbMain.clear();
}

void LMainWindow::OnDropFiles() {
	addFiles(onDropFiles.data);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// PRIVATE FUNCTIONS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void LMainWindow::addFolder(std::string& path, bool recurseSubdirectories) {
	if (recurseSubdirectories) {
		std::vector<std::string> fileList;
		filesInDirectory_recurse(path.c_str(), fileList);
		addFiles( fileList ); // coercive string pooling

	} else {
		addFiles(
			filesInDirectory(path.c_str()),
			spool.pool(path) // faster single string pooling
		);
	}
}

void LMainWindow::addFiles(std::vector<std::string>& files) {
	// Manual string pooling (slower)

	ui->lbMain.setVisible(false);

	for (int i = 0, e = files.size(); i < e; i++) {
		for (int xi = 0, xe = COUNTOF(extensions); xi < xe; xi++) {
			// FIXME: ensure the extension occurs right at the end
			if (files[i].find( extensions[xi]) != files[i].npos) {

				// Get pool index for entry
				//int poolkey = 0;
				size_t spos = files[i].find_last_of(PATH_SEPARATOR);
				int poolkey = spool.pool(files[i].substr(0, spos));
				std::string filename = files[i].substr(spos + 1);

				// Add entry
				unsigned int idx = ui->lbMain.addItem(filename.c_str());
				ui->lbMain.setItemData(idx, (void*) new listItemData(poolkey, filename));
				continue; // xi
			}
		}
	}

	ui->lbMain.setVisible(true);

	repopulateStatusBar();
}

void LMainWindow::addFiles(std::vector<std::string>& files, int poolkey) {
	// Explicit string pooling

	ui->lbMain.setVisible(false);

	for (int i = 0, e = files.size(); i < e; i++) {
		for (int xi = 0, xe = COUNTOF(extensions); xi < xe; xi++) {
			// FIXME: ensure the extension occurs right at the end
			if (files[i].find( extensions[xi]) != files[i].npos) {
				unsigned int idx = ui->lbMain.addItem(files[i].c_str());
				ui->lbMain.setItemData(idx, (void*) new listItemData(poolkey, files[i]));
				continue; // xi
			}
		}
	}

	ui->lbMain.setVisible(true);

	repopulateStatusBar();
}

void LMainWindow::loadPlaylist(std::string& path) {
	std::string content;
	if (! file_get_contents(path, content)) return;

	std::vector<std::string> files;
	explode("\n", content, files);
	addFiles(files); // it handles string pooling
}

void LMainWindow::savePlaylist(std::string& path) {
	// Do this instead of file_put_contents to avoid copying the whole playlist
	//   into memory again (although that would be fine and we do it for the
	//   search dialog anyway)

	FILE* f = NULL;
	_wfopen_s(&f, utf8to16(path.c_str()).c_str(), L"wb");
	if (! f) return;

	for(int i = 0, e = ui->lbMain.getItemCount(); i < e; i++) {
		std::string write = ((listItemData*)ui->lbMain.getItemData(i))->getFullPath();
		fwrite(write.c_str(), write.length(), 1, f);
		fwrite("\n", 1, 1, f);
	}

	fclose(f);
}

void LMainWindow::playFile(unsigned int listIndex) {
	// Close existing file
	//bool bWasPaused = (BASS_ChannelIsActive(hCurrent) == BASS_ACTIVE_PAUSED);
	if (hCurrent) BASS_StreamFree(hCurrent);

	// Assert listIndex within range
	if (listIndex >= ui->lbMain.getItemCount() ) return; // invalid index

	// Retrieve file data
	listItemData* d = (listItemData*) ui->lbMain.getItemData(listIndex);	
	d->m_bAlreadyPlayed = true;
	formatListItemColour(listIndex, d);
	curListIndex = listIndex;
	ui->lbMain.setSelected(listIndex);
	ui->lbMain.focus();	

	// Cosmetic: Set window title
	std::string windowTitle = d->m_fname + " - " APPLICATION_TITLE;
	this->setCaption(windowTitle.c_str());

	// Point BASS toward the file (note: we might still be paused)	
	std::string filepath = d->getFullPath();
	hCurrent = BASS_StreamCreateFile(
		FALSE,
		static_cast<const void*>(utf8to16(filepath.c_str()).c_str()), 0, 0, BASS_UNICODE
	);
	BASS_ChannelSetSync(hCurrent, BASS_SYNC_END | BASS_SYNC_ONETIME, 0, &EventProc, this);
	BASS_ChannelPlay(hCurrent, true);
	// Don't re-pause, what were we thinking?
	//if (bWasPaused) BASS_ChannelPause(hCurrent);

	// Cosmetic: Calculate and store file bitrate
	qwChannelLength = BASS_ChannelGetLength(hCurrent, BASS_POS_BYTE);
	double time = BASS_ChannelBytes2Seconds( hCurrent, qwChannelLength);
	QWORD len  = BASS_StreamGetFilePosition(hCurrent, BASS_FILEPOS_END);
	lBitrate   = (long) (len / (125*time)+0.5);

	// Cosmetic: Cache file type
	BASS_CHANNELINFO cinf;
	BASS_ChannelGetInfo(hCurrent, &cinf);
	if (false) {
	} else if (cinf.ctype == BASS_CTYPE_STREAM_MP3) {		tFormat = ("MP3");
	} else if (cinf.ctype == BASS_CTYPE_STREAM_AAC) {		tFormat = ("AAC");
	} else if (cinf.ctype == BASS_CTYPE_STREAM_MP4) {		tFormat = ("MP4");
	} else if (cinf.ctype == BASS_CTYPE_STREAM_FLAC) {		tFormat = ("FLAC");
	} else if (cinf.ctype == BASS_CTYPE_STREAM_FLAC_OGG) {	tFormat = ("FLAC");
	} else if (cinf.ctype == BASS_CTYPE_STREAM_OGG) {		tFormat = ("VORBIS");	
	} else if (cinf.ctype == BASS_CTYPE_STREAM_ALAC) {		tFormat = ("ALAC");	
	} else if (cinf.ctype == BASS_CTYPE_STREAM_AC3) {		tFormat = ("AC3");
	} else if (cinf.ctype |= BASS_CTYPE_STREAM_WAV) {		tFormat = ("WAV");
	} else {
		tFormat = ("UNKNOWN");
	}

	// Cosmetic: Update display with bitrate + file type
	repopulateStatusBar();
	
}

void LMainWindow::playNext() {	
	if (qIndexes.size()) {

		// Work through queue
		unsigned int newIdx = qIndexes.front();
		qIndexes.pop_front();
		formatQueuedListItems();

		if (newIdx > ui->lbMain.getItemCount()) {
			return; // queue corrupt, shouldn't happen
		}

		listItemData* d = static_cast<listItemData*>(ui->lbMain.getItemData(newIdx));
		d->m_bIsQueued = false;
		formatListItem(newIdx, d);
		playFile(newIdx);

	} else {

		// No items in queue, look at playback mode
		int count = ui->lbMain.getItemCount();
		if (! count) return; // nothing to play

		switch (currentMode) {
			case MODE_REPEATLIST: {
				int newIdx = (curListIndex+1) % count;
				playFile(newIdx);
			} break;

			case MODE_REPEATFILE: {
				playFile(curListIndex);
			} break;

			case MODE_PLAYLIST: {
				if (curListIndex == ui->lbMain.getItemCount()) {
					BASS_ChannelStop(hCurrent);
				} else {
					playFile(curListIndex + 1);
				}
			} break;

			case MODE_PLAYFILE: {
				BASS_ChannelStop(hCurrent);
			} break;

			/*
			case MODE_SHUFFLE: {
				int newIdx = rand() % count;
				playFile(newIdx);
			} break;
			*/

			case MODE_SHUFFLE: {
				// Check if target file has already been played.
				// If we hit too many of these, then reset all already-played markers
#define SHUFFLE_MAX_ALREADY_PLAYED_FAILURES (1000)
				int tries = SHUFFLE_MAX_ALREADY_PLAYED_FAILURES;
				bool found = false;
				while(--tries) {
					int newIdx = random_in_range(0, count);
					if ( ((listItemData*)ui->lbMain.getItemData(newIdx))->m_bAlreadyPlayed ) {
						// already played
					} else {
						playFile(newIdx);
						found = true;
						break /* while */;
					}
				}
				
				if (! found) {
					// We just looped through heaps of files and they've all 
					//  already been played... give up and reset all markers
					ui->lbMain.setVisible(false);
					for (size_t i = 0, e = ui->lbMain.getItemCount(); i != e; ++i) {
						listItemData* lid = (listItemData*) ui->lbMain.getItemData(i);
						lid->m_bAlreadyPlayed = false;
						ui->lbMain.setItemColor(i); // reset color too
					}
					ui->lbMain.refresh();
					ui->lbMain.setVisible(true);

					// Simple shuffle
					int newIdx = random_in_range(0, count);
					playFile(newIdx);
				}

			} break;
			default: {
				// Unknown mode!
				// Stop the music and put the system into a known state
				BASS_ChannelStop(hCurrent);
				setShuffleMode(MODE_REPEATLIST);
			} break;
		}
	}
}

void LMainWindow::setShuffleMode(int newMode) {
	currentMode = newMode;
	tCurrentMode = ui->checkShuffleMenuItem(newMode);
	repopulateStatusBar();
}

void LMainWindow::setDisplayMode(int newMode) {
	// Updating the display mode changes the text, but not the colour of any item.
	displayMode = newMode;

	ui->mnuViewFilenames.setChecked(false);
	ui->mnuViewFullpath.setChecked(false);
	switch(newMode) {
		case DISPLAY_FULLPATH: {
			ui->mnuViewFullpath.setChecked(true);			
		} break;
		case DISPLAY_NAMEONLY: {			
			ui->mnuViewFilenames.setChecked(true);
		} break;
	}

	ui->lbMain.setVisible(false);
	for (int i=0, e = ui->lbMain.getItemCount(); i < e; i++) {
		listItemData* d = static_cast<listItemData*>(ui->lbMain.getItemData(i));
		formatListItem(i, d);
	}
	ui->lbMain.setVisible(true);
}

void LMainWindow::formatListItem(int idx, listItemData* d) {	
	ui->lbMain.setItemText(idx, d->format(displayMode).c_str());
}

void LMainWindow::formatQueuedListItem(int idx, listItemData* d, int queue_pos) {
	std::string strItemName = string_format(("%d > %s"), queue_pos, d->format(displayMode).c_str() );
	ui->lbMain.setItemText(idx, strItemName.c_str());
}

void LMainWindow::formatListItemColour(int idx, listItemData* d) {
	if (d->m_bIsQueued) {
		ui->lbMain.setItemColor(idx, 0xFF, 0x00, 0x00); // Red
	} else if (d->m_bAlreadyPlayed) {
		ui->lbMain.setItemColor(idx, 0x80, 0x80, 0x80); // Grey
	} else {
		ui->lbMain.setItemColor(idx, 0x00, 0x00, 0x00); // Black
	}
}

void LMainWindow::formatQueuedListItems() {
	std::string strItemName;
	int j = 1;
	for(
		qIndexes_t::iterator i = qIndexes.begin(), e = qIndexes.end();
		i != e; i++, j++
	) {
		listItemData* q_item_data = static_cast<listItemData*>(ui->lbMain.getItemData(*i));
		formatQueuedListItem(*i, q_item_data, j);
		formatListItemColour(*i, q_item_data);
	}
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// COSMETIC FUNCTIONS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void LMainWindow::repopulateStatusBar() {	
	std::string tsStatus;
	if (hCurrent) {
		if (BASS_ChannelIsActive(hCurrent) == BASS_ACTIVE_PAUSED) {
			tsStatus = string_format(
				("%d Files        %s        Paused"),
				ui->lbMain.getItemCount(),
				tCurrentMode.c_str()
			);
		} else {
			tsStatus = string_format(
				("%d Files        %s        %s %d kbps"),
				ui->lbMain.getItemCount(),
				tCurrentMode.c_str(),
				tFormat.c_str(),
				lBitrate
			);
		}
	} else {
		tsStatus = string_format(
			("%d Files        %s"),
			ui->lbMain.getItemCount(),
			tCurrentMode.c_str()
		);
	}
	
	this->setStatusBar(tsStatus.c_str());
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// EVENT CALLBACKS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void LMainWindow::OnBASSTimer() {

	// Cache previous values, and only update displays if they have
	//   changed
	static HSTREAM hCurrent_Last = -1;
	static float volume_last = -1;
	//static std::string volume_text_last;

	// Volume
	float volume = BASS_GetVolume();	
	if (volume != volume_last) {		
		std::string tsVolume = string_format("%d%%", (int)(volume * 100));
		ui->llVolume.setText(tsVolume.c_str() );
		volume_last = volume;
	}

	//pbVolume->setValue(volume); //?? FIXME

	// Level sliders
	DWORD curLevel = BASS_ChannelGetLevel(hCurrent);
	ui->pbLevelLeft .setValue( LOWORD(curLevel) / 32768.0f );
	ui->pbLevelRight.setValue( HIWORD(curLevel) / 32768.0f );

	// Play state
	if (hCurrent == 0) {
		// Only redraw if this is the first time we notice it being stopped.
		// Another step towards reducing flicker
		if (hCurrent_Last != 0) {
			ui->llProgress.setText(("Stopped"));
			ui->pbProgress.setValue(0);
		}
	} else {
		QWORD pos = BASS_ChannelGetPosition(hCurrent, BASS_POS_BYTE);
		long len_t = (long)BASS_ChannelBytes2Seconds(hCurrent, qwChannelLength);
		long pos_t = (long)BASS_ChannelBytes2Seconds(hCurrent, pos);

		std::string sProgress = string_format(
			("%d:%02d / %d:%02d / -%d:%02d"),
			pos_t / 60, pos_t % 60,
			len_t / 60, len_t % 60,
			(len_t-pos_t)/60, (len_t-pos_t)%60
		);
		//llProgress->refresh();
		if (sProgress != sCurrentProgress) {
			ui->llProgress.setText( sProgress.c_str() );
			sCurrentProgress = sProgress;
		}

		ui->pbProgress.setValue((float)pos / (float)qwChannelLength);
	}

	hCurrent_Last = hCurrent;
}

void LMainWindow::OnListDoubleClick() {
	int newIdx = ui->lbMain.getSelected();
	if (newIdx != -1) playFile(newIdx);
}

void LMainWindow::OnProgressDragged() {
	if (! hCurrent) return;

	float xMouseCur = (float) LOWORD(ui->pbProgress.onDrag.data);
	float PBWidth   = (float) ui->pbProgress.getInnerWidth();
	float posFraction = xMouseCur / PBWidth;
	BASS_ChannelSetPosition(hCurrent, (QWORD)(posFraction*qwChannelLength), BASS_POS_BYTE);
	ui->pbProgress.setValue(posFraction);
}

void LMainWindow::OnProgressMDown() {
	ui->pbProgress.onDrag.data = ui->pbProgress.onMouseDown.data;
	OnProgressDragged();
}

void LMainWindow::OnWindowResize() {
	ui->resize();
}

void LMainWindow::OnWindowMove() {
	this->snapToScreenEdge();
}

void LMainWindow::OnHotkeyFocusWindow() {
	this->focus();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// MENU COMMANDS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void LMainWindow::OnFileOpenFolder() {
	std::string response;
	bool recurse;
	if (!this->dlgFolderSelectEx("Select a folder to open.", "Include subfolders", response, recurse)) return;

	clear();
	addFolder(response, recurse);
}

void LMainWindow::OnFileAddFolder() {
	std::string response;
	bool recurse;
	if (!this->dlgFolderSelectEx("Select a folder to open.", "Include subfolders", response, recurse)) return;

	addFolder(response, recurse);
}

void LMainWindow::OnFileAddFiles() {
	// TODO
}

void LMainWindow::OnFileExit() {
	this->close();
}

void LMainWindow::OnToolsSelectCurrent() {
	if (hCurrent) {
		ui->lbMain.setSelected(curListIndex);
		ui->lbMain.focus();
	}
}

void LMainWindow::findFormCommon(MGKeyCombination kc, bool force) {
	std::string basekey;
	if (kc.virtkey == MGVK_NONE) {
		// basekey is blank, shouldn't trigger the find form unless this was
		//  a specific action request
		if (! force) {
			return;
		}
	} else if (kc.virtkey >= MGVK_0 && kc.virtkey <= MGVK_Z && !kc.alt && !kc.ctrl) {
		basekey = MGKC_FormatCharacter(kc);
	} else {
		// The virtkey shouldn't trigger the find form.
		return;
	}

	LFindWindow lfw(this);
	if (basekey.size()) lfw.setSearchText(basekey.c_str());
	lfw.doEventLoop();

	unsigned selidx = lfw.finalSelection;
	if (selidx != NOTHING_SELECTED) {
		playFile(selidx);
	}
};

void LMainWindow::OnEditFind() { findFormCommon(MGKeyCombination(), true); }
void LMainWindow::OnKeypress_form() {
	onKey.event_preventDefault();

	findFormCommon(onKey.data, false);
}
void LMainWindow::OnKeypress_lb() {
	ui->lbMain.onKey.event_preventDefault();

	findFormCommon(ui->lbMain.onKey.data, false);
}

void LMainWindow::OnEditExplore() {
	unsigned selidx = ui->lbMain.getSelected();
	if (selidx == NOTHING_SELECTED) {
		return;
	}
	listItemData *lid = (listItemData*)ui->lbMain.getItemData( selidx );

	// 
#ifdef WIN32
	exec_nowait(
		"C:\\windows\\explorer.exe",
		// Windows filenames can't include a ", or otherwise this would need
		//  escaping
		(std::string("/select,\"")+lid->getFullPath()+"\"").c_str()
	);
#else
	// no-op
#endif
}

void LMainWindow::OnEditDelete() {
	std::vector<unsigned int> selidxs;
	ui->lbMain.getAllSelected(selidxs);
	if (! selidxs.size()) return;

	if (selidxs.size() == ui->lbMain.getItemCount()) {
		for(size_t i = 0, e = selidxs.size(); i != e; ++i) {
			listItemData* lid = (listItemData*)ui->lbMain.getItemData(i);
			delete lid;
		}
		ui->lbMain.clear();
		qIndexes.clear();		
		
	} else {
		// Remove selected items

		for(unsigned int i = 0, e = selidxs.size(); i != e; ++i) {
			unsigned int idx = selidxs[i];

			// Scan for whether it's in the queue, if so remove it
		
			qIndexes_t::iterator ii = qIndexes.begin();
			while(ii != qIndexes.end()) {
				if (*ii == idx) {				
					// Remove from qIndexes
					ii = qIndexes.erase(ii); // ++

					// Remove from being in the queue
					listItemData* lid = (listItemData*) ui->lbMain.getItemData(idx);
					lid->m_bIsQueued = false;

					// Also re-render this specific item
					formatListItem(idx, lid);
					formatListItemColour(idx, lid);
				} else {
					++ii;
				}
			}

			// Remove the list item + free memory
			listItemData* lid = (listItemData*)ui->lbMain.getItemData(idx);
			delete lid;
			ui->lbMain.removeItem(idx);

			// Move all successive queue items one place further forward

			for(qIndexes_t::iterator ii = qIndexes.begin(), ee = qIndexes.end(); ii != ee; ++ii) {
				//unsigned int val = *ii;
				if (*ii > idx) {
					*ii -= 1;//val - 1;
				}
			}

			// Move all successive delete-selection indices one place further forward
			for(size_t ii = i; ii != e; ++ii) {
				if (selidxs[ii] > idx) {
					selidxs[ii] -= 1;
				}
			}
		}

	}

	// Re-render queued items
	formatQueuedListItems();
	ui->lbMain.refresh(); // necessary?
}

void LMainWindow::OnPlayPlay() {
	unsigned int selIdx = ui->lbMain.getSelected();
	if (selIdx == NOTHING_SELECTED) {
		playNext();
	} else {
		playFile(selIdx);
	}
}

void LMainWindow::OnPlayPause() {
	long status = BASS_ChannelIsActive(hCurrent);
	switch (status) {
		case BASS_ACTIVE_STOPPED: {
			// Either audio stopped, or hCurrent is null/invalid. "Super Unpause"
			playNext();
		} break;
		case BASS_ACTIVE_PAUSED: {
			BASS_ChannelPlay(hCurrent, false);
		} break;
		default: {
			BASS_ChannelPause(hCurrent);
		} break;
	}
	repopulateStatusBar();
}

void LMainWindow::OnPlayQueue() {
	//int sel_idx = ui->lbMain.getSelected();
	std::vector<unsigned int> idxs;
	ui->lbMain.getAllSelected(idxs);

	for(unsigned int ii = 0, ee = idxs.size(); ii != ee; ++ii) {
		unsigned int sel_idx = idxs[ii];

		// Check if it already exists in the queue..
		bool bFound = false;
		qIndexes_t::iterator i = qIndexes.begin(), e = qIndexes.end();
		for (; i != e; i++) {
			if (sel_idx == *i) {
				bFound = true;
				break;
			}
		}
	
		// ..and if so, remove it
		listItemData* item_data = static_cast<listItemData*>(ui->lbMain.getItemData(sel_idx));
		if (bFound) {		
			item_data->m_bIsQueued = false;
			formatListItem(*i, item_data);
			formatListItemColour(*i, item_data);
			qIndexes.erase(i);
		} else {		
			item_data->m_bIsQueued = true;
			qIndexes.push_back(sel_idx);
		}

	}

	// Redraw queued items in list
	formatQueuedListItems();

	// Select the following item (for queue slides)
	int count = ui->lbMain.getItemCount();
	//ui->lbMain.setSelected(sel_idx+1 > count-1 ? count-1 : sel_idx+2); // FIXME: very wrong
}

void LMainWindow::OnPlayStop() {
	long status = BASS_ChannelIsActive(hCurrent);
	switch (status) {
		case BASS_ACTIVE_PLAYING: {
			BASS_ChannelPause(hCurrent);
		} break;
		default: {
			BASS_ChannelStop(hCurrent);
		} break;
	}
	repopulateStatusBar();
}

void LMainWindow::OnViewFilenames() {
	setDisplayMode(DISPLAY_NAMEONLY);
}

void LMainWindow::OnViewFullpath() {
	setDisplayMode(DISPLAY_FULLPATH);	
}

void LMainWindow::OnViewToggle() {
	setDisplayMode((displayMode + 1) % DISPLAY_COUNT_MODES);
}

void LMainWindow::OnOptionsSettings() {
	LSettingsWindow wndSettings(this);
	wndSettings.doEventLoop();
	// Heap destructor
}

void LMainWindow::OnOptionsRepeatList() {
	setShuffleMode(MODE_REPEATLIST);
}

void LMainWindow::OnOptionsRepeatFile() {
	setShuffleMode(MODE_REPEATFILE);
}

void LMainWindow::OnOptionsPlayList() {
	setShuffleMode(MODE_PLAYLIST);
}

void LMainWindow::OnOptionsPlayFile() {
	setShuffleMode(MODE_PLAYFILE);
}

void LMainWindow::OnOptionsShuffle() {
	setShuffleMode(MODE_SHUFFLE);
}

void LMainWindow::OnOptionsToggle() {
	setShuffleMode((currentMode + 1) % MODE_COUNT_MODES);
}

void LMainWindow::OnHelpAbout() {
	std::string msg =
		"Lilly is a fast no nonsense music player.\n" \
		"Learn the shortcuts and go!\n\n" \
		"Build date: " + getLinkTimeString()
	;
	this->dlgAlert(msg.c_str(), APPLICATION_TITLE);
}
