// LMainWindow.h
// `````````````

#pragma once
#include "stdafx.h"
#include "LSettingsWindow.h"
#include "LFindWindow.h"

struct listItemData;
struct LMainWindowUI;

class LMainWindow : public MGWindow {
private:

	// UI

	struct LMainWindowUI* ui;
public: // Modified by LSettingsWindow
	MGHotkey      *hkNextSong, *hkPlayPause, *hkFocusWindow;
private:

	// BASSlib handles for currently playing file/channel

	HSTREAM hCurrent;
	long    lBitrate;
	std::string tFormat;
	QWORD   qwChannelLength;
	int     curListIndex;

	// File switching and display modes	

	int     displayMode;
	int		currentMode;
	std::string tCurrentMode;

#if (__cplusplus > 199711L)
	// Save some memory, use c++11 singly-linked list instead
	typedef std::forward_list<unsigned int> qIndexes_t;
#else
	typedef std::list<unsigned int> qIndexes_t;
#endif
	qIndexes_t qIndexes;
	std::string sCurrentProgress;

	MGTimer       *tmrBASS;
	
	// Functions

public:
	MGListBox* exposeListBox();

	void addFolder(std::string& path, bool recurseSubdirectories);
	void addFiles(std::vector<std::string>& files);
	void addFiles(std::vector<std::string>& files, int poolkey);
	void savePlaylist(std::string& path);
	void loadPlaylist(std::string& path);

	void playFile(unsigned int listIndex);
	void playNext();
	void clear();

private:
	void setShuffleMode(int newMode);
	void setDisplayMode(int newMode);
	void formatListItem(int idx, listItemData* d);
	void formatListItemColour(int idx, listItemData* d);
	void formatQueuedListItem(int idx, listItemData* d, int queue_pos);	
	void formatQueuedListItems();
	void repopulateStatusBar();

	void findFormCommon(MGKeyCombination kc, bool force);

	// Event callbacks

	void OnWindowResize();
	void OnWindowMove();
	void OnWindowClosing();
	void OnBASSTimer();
	void OnListDoubleClick();
	void OnProgressDragged();
	void OnProgressMDown();
	void OnHotkeyFocusWindow();
	void OnKeypress_form();
	void OnKeypress_lb();
	void OnDropFiles();

	// Menu commands
	// Public as they're assigned in the constructor of LMainWindowUI

public:
	void OnFileOpenFolder();
	void OnFileAddFolder();
	void OnFileAddFiles();
	void OnFileExit();
	void OnToolsSelectCurrent();	
	void OnEditDelete();
	void OnEditExplore();
	void OnEditFind();
	void OnPlayPlay();
	void OnPlayPause();
	void OnPlayStop();
	void OnPlayQueue();
	void OnViewFilenames();
	void OnViewFullpath();
	void OnViewToggle();
	void OnOptionsSettings();
	void OnOptionsRepeatList();
	void OnOptionsRepeatFile();
	void OnOptionsPlayList();
	void OnOptionsPlayFile();
	void OnOptionsShuffle();
	void OnOptionsToggle();
	void OnHelpAbout();

public:
	
	// Constructor / Deconstructor

	LMainWindow();
	~LMainWindow();
	
	// Public functions

	//void doChildWindowEvents();	
	static void CALLBACK EventProc(HSYNC handle, DWORD channel, DWORD data, void* user);

};