#pragma once
#ifndef LMAINWINDOW_INTERNAL
#error "Only include this file from LMainWindow!"
#endif

struct LMainWindowUI {

	// Parent window

	LMainWindow* p;

	// Controls

	MGListBox     lbMain;
	MGProgressBar pbProgress, /*pbVolume,*/ pbLevelLeft, pbLevelRight;
	MGLabel       llProgress, llVolume;

	// Menu items

	MGMenuItem
		mnuFile,
			mnuFileOpenFolder, mnuFileAddFolder, mnuFileAddFiles,
			mnuFileOpenPlaylist, mnuFileSavePlaylist,
			mnuFileExit,
		mnuEdit,
			mnuEditProperties, mnuEditRemove, mnuEditDelete, mnuEditExplore,
			mnuEditTools,
				mnuTSelectCurrent,
				mnuTClearAll,
			mnuEditFind,
		mnuPlay,
			mnuPlayPlay, mnuPlayPause, mnuPlayStop,
			mnuPlayNext, mnuPlayPrevious, mnuPlayQueue,
		mnuFavorites,
			mnuFavsAdd,
		mnuOptions,
			mnuOptSettings,
			mnuOptRepeatList, mnuOptRepeatFile, mnuOptPlayList, mnuOptPlayFile,
			mnuOptShuffle,    mnuOptToggle,
		mnuView,
			mnuViewFilenames, mnuViewFullpath, mnuViewRelpath,
			mnuViewToggle,
			mnuViewControls, mnuViewProgress,
		mnuHelp,
			mnuHelpOverview, mnuHelpHomepage,
			mnuHelpAbout
	;

	LMainWindowUI(LMainWindow* p) :
		p(p),

		// Form controls

		lbMain(p, false, true, 1, true),
		pbProgress(p, true, true),
		pbLevelLeft(p, false, false),
		pbLevelRight(p, false, false),
		llProgress(p, true),
		llVolume(p, false),

		// Menu items

		mnuFile(p, "&File"),
			mnuFileOpenFolder(&mnuFile, "Open folder...", MGCB(LMainWindow::OnFileOpenFolder), MGKC(MGVK_F4, false, false, false)),
			mnuFileAddFolder(&mnuFile, "&Add folder...",		MGCB(LMainWindow::OnFileAddFolder), MGKC(MGVK_F3, false, false, false)),
			mnuFileAddFiles(&mnuFile, "Add &file(s)...", MGCB(LMainWindow::OnFileAddFiles), MGKC(MGVK_L, false, true, false)),
			//mnuFile->addChildMenuSeparator(),
			mnuFileOpenPlaylist(&mnuFile, "O&pen playlist..."), // C-O
			mnuFileSavePlaylist(&mnuFile, "&Save playlist..."), // C-S
			//mnuFile->addChildMenuSeparator(),
			mnuFileExit            (&mnuFile, "E&xit",			MGCB(LMainWindow::OnFileExit), MGKC(MGVK_ESC, false, false, false)),
		mnuEdit                (p, "&Edit"),
			mnuEditProperties      (&mnuEdit, "&Properties..."), // F2
			mnuEditRemove          (&mnuEdit, "&Remove",  MGCB(LMainWindow::OnEditDelete), MGKC(MGVK_DEL, false, false, false)), // Del
			mnuEditDelete          (&mnuEdit, "&Delete..."), // S-Del
			mnuEditExplore         (&mnuEdit, "&Explore...", MGCB(LMainWindow::OnEditExplore), MGKC(MGVK_E, false, true, false)),
			mnuEditTools           (&mnuEdit, "&Tools"),
				mnuTSelectCurrent      (&mnuEditTools, "Select &playing item", MGCB(LMainWindow::OnToolsSelectCurrent), MGKC(MGVK_TILDE, false, false, false)),
				mnuTClearAll           (&mnuEditTools, "C&lear list", MGCB(LMainWindow::clear), MGKC(MGVK_N, false, true, false)),
			//mnuEdit->addChildMenuSeparator(),
			mnuEditFind            (&mnuEdit, "&Find...", MGCB(LMainWindow::OnEditFind), MGKC(MGVK_F, false, true, false)),
		mnuPlay                (p, "&Play"),
			mnuPlayPlay            (&mnuPlay, "&Play",			MGCB(LMainWindow::OnPlayPlay), MGKC(MGVK_ENTER, false, false, false)),
			mnuPlayPause           (&mnuPlay, "Pa&use",			MGCB(LMainWindow::OnPlayPause), MGKC(MGVK_SPACE, false, false, false)),
			mnuPlayStop            (&mnuPlay, "&Stop",              MGCB(LMainWindow::OnPlayStop), MGKC(MGVK_0, false, false, false)),
			//mnuPlay->addChildMenuSeparator(),
			mnuPlayNext            (&mnuPlay, "&Next",			MGCB(LMainWindow::playNext), MGKC(MGVK_TAB, false, false, false)),
			mnuPlayPrevious        (&mnuPlay, "P&revious"),
			mnuPlayQueue	       (&mnuPlay, "&Queue",           MGCB(LMainWindow::OnPlayQueue), MGKC(MGVK_INS, false, false, false)),
		mnuFavorites           (p, "F&avorites"),
			mnuFavsAdd             (&mnuFavorites, "&Add playlist to favorites..."), // C-D
		mnuOptions             (p, "&Options"),
			mnuOptSettings         (&mnuOptions, "&Settings...",   MGCB(LMainWindow::OnOptionsSettings), MGKC(MGVK_F6, false, false, false)),
			//mnuOptions->addChildMenuSeparator(),
			mnuOptRepeatList       (&mnuOptions, "&Repeat list",MGCB(LMainWindow::OnOptionsRepeatList), MGKC(MGVK_N, true, false, false)),
			mnuOptRepeatFile       (&mnuOptions, "R&epeat file",MGCB(LMainWindow::OnOptionsRepeatFile), MGKC(MGVK_E, true, false, false)),
			mnuOptPlayList         (&mnuOptions, "&Play list",  MGCB(LMainWindow::OnOptionsPlayList), MGKC(MGVK_L, true, false, false)),
			mnuOptPlayFile         (&mnuOptions, "P&lay file",  MGCB(LMainWindow::OnOptionsPlayFile), MGKC(MGVK_F, true, false, false)),
			mnuOptShuffle          (&mnuOptions, "S&huffle",    MGCB(LMainWindow::OnOptionsShuffle), MGKC(MGVK_S, true, false, false)),
			mnuOptToggle           (&mnuOptions, "&Toggle",        MGCB(LMainWindow::OnOptionsToggle), MGKC(MGVK_F9, false, false, false)),
		mnuView                (p, "&View"),
			mnuViewFilenames       (&mnuView, "View as &filename",		MGCB(LMainWindow::OnViewFilenames)),
			mnuViewFullpath        (&mnuView, "View as full &path",		MGCB(LMainWindow::OnViewFullpath)),
			mnuViewRelpath         (&mnuView, "View as &relative path"),
			mnuViewToggle          (&mnuView, "&Toggle",            MGCB(LMainWindow::OnViewToggle)),
			//mnuView->addChildMenuSeparator(),
			mnuViewControls        (&mnuView, "&Controls"),
			mnuViewProgress        (&mnuView, "Pr&ogress"),
		mnuHelp                (p, "&Help"),
			mnuHelpOverview        (&mnuHelp, "&Overview..."), // F1
			mnuHelpHomepage        (&mnuHelp, "Visit &homepage..."), // C-F1
			//mnuHelp->addChildMenuSeparator(),
			mnuHelpAbout           (&mnuHelp, "&About...",				MGCB(LMainWindow::OnHelpAbout))

		// Specific initialisation
	{
		pbProgress.removeStyles();

		pbLevelLeft.setValue(0);
		pbLevelLeft.removeStyles();
		pbLevelLeft.removeBorder();

		pbLevelRight.setValue(0);
		pbLevelRight.removeStyles();
		pbLevelRight.removeBorder();

	};

	void postMenuInitialisation() {
		
		mnuFileAddFiles.setEnabled(false);
		mnuEditDelete.setEnabled(false);
		mnuPlayPrevious.setEnabled(false);
		mnuViewRelpath.setEnabled(false);
		mnuViewProgress.setChecked(true);
		mnuViewControls.setEnabled(false);
		mnuHelpOverview.setEnabled(false);
		mnuHelpHomepage.setEnabled(false);
	};

	void resize() {
		p->snapToScreenEdge();

		int h = p->getInnerHeight();
		int w = p->getInnerWidth();

		pbProgress.resize(2, 0,  w - 24, 16);
		llProgress.resize(4, 18, w - 64, 16);

		pbLevelLeft.resize(w - 20, 0,  10, 34);
		pbLevelRight.resize(w - 11, 0,  10, 34);
		llVolume.resize(w - 64, 18, 40, 16);

		lbMain.resize(0, 36, w, h - 36);
	};

	~LMainWindowUI() {};

	// Helpers

	std::string checkShuffleMenuItem(int mode) {
		mnuOptRepeatList.setChecked(false);
		mnuOptRepeatFile.setChecked(false);
		mnuOptPlayList.setChecked(false);
		mnuOptPlayFile.setChecked(false);
		mnuOptShuffle.setChecked(false);
		switch (mode) {
			case MODE_REPEATLIST: {
				mnuOptRepeatList.setChecked(true);
				return ""; // Repeat list: intentionally blank
			} break;
			case MODE_REPEATFILE: {
				mnuOptRepeatFile.setChecked(true);
				return "Repeat file";
			} break;
			case MODE_PLAYLIST: {
				mnuOptPlayList.setChecked(true);
				return "Play list";
			} break;
			case MODE_PLAYFILE: {
				mnuOptPlayFile.setChecked(true);
				return "Play file";
			} break;
			case MODE_SHUFFLE: {
				mnuOptShuffle.setChecked(true);
				return "Shuffle";
			} break;
		}
		return "Unknown mode";
	};
};
