// LSettingsWindow.cpp
// ```````````````````

#include "stdafx.h"
#include "LMainWindow.h"
#include "LSettingsWindow.h"
#include "res/resource.h"

LSettingsWindow::LSettingsWindow(MGWindow* parent) :
MGWindow((void*)IDI_LILLYMAIN, ("Settings"), parent)
{
	this->setResizable(false);
	this->resize(400, 400);

	lbOptions = new MGListBox(this, false, false, 1, false);
	lbOptions->resize(7, 7, 100, this->getInnerHeight() - 14);
	lbOptions->addItem(("Setup"));
	lbOptions->addItem(("Media"));
	lbOptions->addItem(("Hotkeys"));
	lbOptions->addItem(("Device"));
	lbOptions->onChange.assign(this, MGCB(LSettingsWindow::onListChanged));
	lbOptions->setSelected(0);

	grSettings = new MGGroup(this);
	grSettings->resize(114, 7, this->getInnerWidth() - 114-7, this->getInnerHeight() -14);
	
	// Load hotkeys
	hotkeys.push_back(std::make_pair("Next Song", static_cast<LMainWindow*>(parent)->hkNextSong));
	hotkeys.push_back(std::make_pair("Play/Pause", static_cast<LMainWindow*>(parent)->hkPlayPause));
	hotkeys.push_back(std::make_pair("Focus Window", static_cast<LMainWindow*>(parent)->hkFocusWindow));

	// Hotkeys tab
	
	lbHotkeys = new MGListBox(this, false, false, 1, false);
	lbHotkeys->resize(125, 28, 250, 100);
	lbHotkeys->onChange.assign(this, MGCB(LSettingsWindow::onHotkeySelected));
	repopulateHotkeys();
	hcKeyinput = new MGHotkeyControl(this);
	hcKeyinput->resize(125, 140, 250, 20);
	btSetHotkey = new MGButton(this, ("Assign"));
	btSetHotkey->resize(125, 165, 80, 22); 
	btSetHotkey->onClick.assign(this, MGCB(LSettingsWindow::onHotkeyAssign));

	// Device tab

	lDevices = new MGLabel(this, true);
	lDevices->resize(125, 28, 250, 20);
	lDevices->setText(("Output device:"));
	cbDevices = new MGComboBox(this, false);
	cbDevices->resize(125, 46, 250, 100);
	cbDevices->allowTextEntry(false);
	repopulateDevices();
	cbDevices->onChange.assign(this, MGCB(LSettingsWindow::onDeviceChanged));
	lSoundInfo = new MGLabel(this, true);
	lSoundInfo->resize(125, 80, 250, 100);
	repopulateSoundInfo();

	// Initial state
	onListChanged();

	// Showtime
	this->setVisible(true);
}

LSettingsWindow::~LSettingsWindow() {
	delete lbOptions;
	delete grSettings;

	delete lbHotkeys;
	delete hcKeyinput;

	delete lDevices;
	delete cbDevices;
	delete lSoundInfo;
	// TODO
}

void LSettingsWindow::onHotkeySelected() {
	unsigned int selidx = lbHotkeys->getSelected();
	if (selidx != -1 && selidx < hotkeys.size()) {
		hcKeyinput->setValue( hotkeys[selidx].second->getCombination() );
	} else {
		hcKeyinput->setValue( MGKC(MGVK_NONE, 0, 0, 0) );
	}
}

void LSettingsWindow::onHotkeyAssign() {
	unsigned int selidx = lbHotkeys->getSelected();
	if (selidx != -1 && selidx < hotkeys.size()) {
		hotkeys[selidx].second->setCombination(
			hcKeyinput->getValue()
		);
	}
	repopulateHotkeys();
	lbHotkeys->setSelected(selidx);
}

void LSettingsWindow::repopulateHotkeys() {
	lbHotkeys->clear();
	for (int i = 0, e = hotkeys.size(); i < e; i++) {
		lbHotkeys->addItem(
			string_format(
				"%s\t%s",
				hotkeys[i].first.c_str(),
				MGKC_Format((hotkeys[i].second)->getCombination()).c_str()
			).c_str()
		);
	}
}

void LSettingsWindow::repopulateDevices() {
	// Load all devices
	BASS_DEVICEINFO info;
	int device_idx = 0;
	for(;;) {
		if (! BASS_GetDeviceInfo(device_idx, &info)) break;
		//TSTRING devName = TSTRINGify(info.name);
		cbDevices->addItem(info.name); //(TCHAR*) devName.c_str());
		device_idx++;
	}
	// Select the current device
	cbDevices->setSelected( BASS_GetDevice()  );
}

void LSettingsWindow::onDeviceChanged() {
	int curIdx = cbDevices->getSelected();
	//FIXME: SetDevice requires BASS_Init called on the old device first / invalidate all
	// current streams - LOW PRIORITY FEATURE
	//BASS_SetDevice( curIdx );
	repopulateSoundInfo();
}

#define UTF8_BULLET "\xE2\x80\xA2"

void LSettingsWindow::repopulateSoundInfo() {
	BASS_INFO binf;
	BASS_GetInfo(&binf);

	std::string sinf = string_format(
		"Output properties:\n" \
		UTF8_BULLET " Master Volume:\t%.1f %%\n" \
		UTF8_BULLET " Frequency:\t%d Hz\n" \
		UTF8_BULLET " Channels:\t%d\n" \
		UTF8_BULLET " Latency:\t%d ms"
		, (BASS_GetVolume()*100), binf.freq, binf.speakers, binf.latency );
	lSoundInfo->setText(/*(TCHAR*)*/ sinf.c_str());
}

void LSettingsWindow::onListChanged() {
	// Hide all control elements
	lbHotkeys->setVisible(false);
	hcKeyinput->setVisible(false);
	btSetHotkey->setVisible(false);

	lDevices->setVisible(false);
	cbDevices->setVisible(false);
	lSoundInfo->setVisible(false);

	// Show relevant control elements
	switch(lbOptions->getSelected()) {
		case 0: { // Setup
			grSettings->setText(("Setup"));
		} break;
		case 1: { // Media
			grSettings->setText(("Media"));
		} break;
		case 2: { // Hotkeys
			grSettings->setText(("Hotkeys"));
			lbHotkeys->setVisible(true);
			hcKeyinput->setVisible(true);
			btSetHotkey->setVisible(true);
		} break;
		case 3: { // Device
			grSettings->setText(("Device"));
			lDevices->setVisible(true);
			cbDevices->setVisible(true);
			lSoundInfo->setVisible(true);
		} break;
	}
}
