// Lilly.cpp
// `````````

/*
 * Memory usage is slightly larger than billy..
 * .. it's probably inherent from using utf16 wchars, STL and the SH folder browser
 */

#include "stdafx.h"
#include "LMainWindow.h"
#include "SettingsManager.h"

#ifdef _WIN32
#define WARN(x) MessageBox(0, L ## x , L"Lilly" , MB_OK)
#else
#define WARN(x)
#endif

int appInit() {

	// Load BASS.DLL

	if (HIWORD(BASS_GetVersion()) != BASSVERSION) {
		WARN("ERROR: Found an incorrect version of bass.dll!");
		return 1;
	}
	if ( ! BASS_Init(-1, 44100, 0, 0, 0) ) {
		WARN("ERROR: Couldn't initialise audio!");
		return 1;
	}

	// Attempt to load format plugins

	std::string pluginList = SettingsManager::getInst()->getString(LS_BassPlugins);
	std::vector<std::string> plugins;
	explode(",", pluginList, plugins);

	std::vector<HPLUGIN> loadedPlugins;

	for(unsigned int i = 0, e = plugins.size(); i != e; ++i) {
		trim(plugins[i]);
		if (plugins[i].size()) {
			std::wstring pname = utf8to16(plugins[i].c_str());
			loadedPlugins.push_back(BASS_PluginLoad(
				// Add flag to indicate we are casting wchar_t to char
				(const char*) pname.c_str(), BASS_UNICODE
			));
		}
	}
	
	BASS_Start();

	// Window lifecycle
	
	LMainWindow* app = new LMainWindow();
	app->doEventLoop();
	/*
	while ( app->isActive() ) {
		app->doEvents();
		app->doChildWindowEvents();
	}
	*/
	delete app; app = 0;
	SettingsManager::getInst()->commit();

	// Tidy up BASS

	BASS_Stop();
	for (int i = 0, e = loadedPlugins.size(); i != e; ++i) {
		BASS_PluginFree(loadedPlugins[i]);
	}
	BASS_Free();

	return 0;
}

#ifdef _WIN32
int APIENTRY wWinMain(HINSTANCE, HINSTANCE, LPTSTR, int) {		
	srand(GetTickCount());
	return appInit();
}
#else
int main(int argc, char** argv) {
	srand(time(NULL));
	return appInit();
}
#endif
