// SettingsManager.cpp
// ```````````````````

#include "stdafx.h"
#include "SettingsManager.h"

#define KEYSETTINGS(x) \
	"Key" x "VK", "Key" x "Alt", "Key" x "Ctrl", "Key" x "Shift"

static const char* SettingKeys[] = {
	"WindowXPos",
	"WindowYPos",
	"WindowWidth",
	"WindowHeight",
	KEYSETTINGS("Next"),
	KEYSETTINGS("PlayPause"),
	KEYSETTINGS("Focus"),
	"BassPlugins"
};

#undef KEYSETTINGS

const int SettingDefaults[] = {
	0, 0, 480, 240,
	MGVK_S, 1, 0, 1,
	MGVK_D, 1, 0, 1,
	MGVK_F, 1, 0, 1
};

// Construction / Deconstruction (RAII)
// ====================================

SettingsManager::SettingsManager() {
	std::string szConfFile( APPLICATION_TITLE CONF_EXTENSION );
	inidata = ini_read(szConfFile);

	data.resize(LS_MAXSETTINGS, 0);
	for (int i = 0; i < LS_MAXSETTINGS; i++) {
		data[i] = _get( SettingKeys[i], SettingDefaults[i] );
	}
}

SettingsManager::~SettingsManager() {
	this->commit();
	if (inst != 0) delete inst;
}

// Singleton acquisition
// =====================

SettingsManager* SettingsManager::inst = 0;
SettingsManager* SettingsManager::getInst() {
	if (inst == 0) inst = new SettingsManager();
	return inst;
}

// Methods
// =======

int SettingsManager::get(enum LillySettings ls) {
	if (ls < 0 || ls >= LS_MAXSETTINGS) return 0;
	return data[ls];
}

std::string SettingsManager::getString(enum LillySettings ls) {
	return inidata[APPLICATION_TITLE][ SettingKeys[ls] ];
}

void SettingsManager::set(enum LillySettings ls, int value) {
	if (ls < 0 || ls >= LS_MAXSETTINGS) return;
	// fast cache
	data[ls] = value;

	// slow cache
	_set( SettingKeys[ls], value);
}

int SettingsManager::_get(std::string key, int default_int) {
	ini_t::const_iterator i = inidata.find(APPLICATION_TITLE);
	if (i == inidata.end()) {
		_set(key, default_int);
		return default_int;
	}

	std::map<std::string,std::string>::const_iterator j = i->second.find(key);
	if (j == i->second.end()) {
		_set(key, default_int);
		return default_int;
	}

	return atoi(j->second.c_str());
}

void SettingsManager::_set(std::string key, int val) {
	char val_str[64];
	sprintf_s(val_str, "%d", val);
	inidata[APPLICATION_TITLE][key] = std::string(val_str);
}

void SettingsManager::commit() {
	std::string szConfFile( APPLICATION_TITLE CONF_EXTENSION );
	ini_write(szConfFile, inidata);
}
