/*
 * Copyright 2012, the mGui Authors. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 *  1. Redistributions of source code must retain the above copyright notice and this
 *     list of conditions.
 *  2. Redistributions in binary form must reproduce the above copyright notice and this
 *     list of conditions in the documentation and/or other materials provided with the
 *     distribution.
 */

#include "mGui_RichEdit.h"

#define MGUI_WIN32_INTERNAL_USE
#include "mGui_Win32.h"

#include <Richedit.h>

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// MGRichEdit
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

MGRichEdit::MGRichEdit(MGWindow* parent, bool bReadOnly) {
	this->_controlType = MGCT_RICHEDIT;
	d = new MGControlData;
	parent->_prepareChild(d, this);
	
	if ( !LoadLibraryW(L"riched20.dll") ) {
		parent->dlgAlert("Couldn't load richedit20.dll!");
		return;
	}
	
	d->hWnd = CreateWindowExW(WS_EX_STATICEDGE, L"RichEdit20W", 0, 
		ES_MULTILINE | ES_NOHIDESEL | WS_VISIBLE | WS_CHILD |
		WS_TABSTOP | WS_VSCROLL | (bReadOnly?ES_READONLY:0),
		0, 0, 1, 1, d->ParentHWnd, (HMENU)d->ID, d->hInstance, NULL);
	if (!d->hWnd) return;

	SelSetToEnd();
	SelFontSet(RGB(0, 0, 0), false, false, false);
}

void MGRichEdit::Clear() {
	SendMessageW(d->hWnd, WM_SETTEXT, 0, (LPARAM) L"" );
}

std::string MGRichEdit::SelTextGet() {
	TEXTRANGE tr; CHARRANGE cr;
	SendMessage(d->hWnd, EM_EXGETSEL, 0, (LPARAM)&cr);
	tr.chrg = cr;

	wchar_t* buff = new wchar_t[cr.cpMax - cr.cpMin];

	//memset(outbuff, 0, charcount * sizeof(TCHAR));
	// Request selected text
	tr.lpstrText = buff;
	SendMessage(d->hWnd, EM_GETTEXTRANGE, 0, (LPARAM)&tr);

	std::string ret = utf16to8(buff);
	delete[] buff;
	return ret;
}

void MGRichEdit::SelTextSet(const char* szText) {
	SendMessage(d->hWnd, EM_REPLACESEL, TRUE, (LPARAM)(utf8to16(szText).c_str()));
}

void MGRichEdit::SelSetRange(LONG lMin, LONG lMax) {
	SendMessage(d->hWnd, EM_SETSEL, (WPARAM)lMin, (LPARAM)lMax);
}

int  MGRichEdit::SelGetStart() {
	CHARRANGE cr;
	SendDlgItemMessage(d->ParentHWnd, d->ID, EM_EXGETSEL, 0, (LPARAM)&cr);
	return cr.cpMin;
}

int  MGRichEdit::SelGetEnd() {
	CHARRANGE cr;
	SendDlgItemMessage(d->ParentHWnd, d->ID, EM_EXGETSEL, 0, (LPARAM)&cr);
	return cr.cpMax;
}

void MGRichEdit::SelSetToEnd() {
	SendMessage(d->hWnd, EM_SETSEL, (WPARAM) -1, (LPARAM) -1);
}

void MGRichEdit::SelFontSet(
	long rgb, bool bBold, bool bIta, bool bUL, const char* szFont, long size
) {
	CHARFORMAT cf = {0};
	cf.cbSize = sizeof(CHARFORMAT);
	SendMessage(d->hWnd, EM_GETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	cf.dwEffects = (bBold?CFE_BOLD:0) | (bIta?CFE_ITALIC:0) | (bUL?CFE_UNDERLINE:0);
	cf.crTextColor = (COLORREF)rgb;
	cf.dwMask = CFM_BOLD | CFM_ITALIC | CFM_UNDERLINE | CFM_COLOR | CFM_FACE | CFM_SIZE;
	wcscpy_s(cf.szFaceName, LF_FACESIZE-1, utf8to16(szFont).c_str());
	cf.yHeight = size;
	SendMessage(d->hWnd, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
}

void MGRichEdit::SelFontSet(long rgb, bool bBold, bool bIta, bool bUL) {
	MGFontManager *fontman = MGFontManager::getInstance();
	SelFontSet(
		rgb, bBold, bIta, bUL, utf16to8(fontman->getFontFace()).c_str(), fontman->getFontTwipHeight()
	);
}

int  MGRichEdit::TextLength() {
	GETTEXTLENGTHEX gtl;
	gtl.codepage = 1200; // Common wchar_t UTF16-LE codepage, as per msdn
	gtl.flags = GTL_CLOSE | GTL_NUMCHARS;
	return (int) SendMessage(d->hWnd, EM_GETTEXTLENGTHEX, (WPARAM) &gtl, 0);
}
