/*
 * Copyright 2012, the mGui Authors. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 *  1. Redistributions of source code must retain the above copyright notice and this
 *     list of conditions.
 *  2. Redistributions in binary form must reproduce the above copyright notice and this
 *     list of conditions in the documentation and/or other materials provided with the
 *     distribution.
 */

#ifndef MGUI_WIN32_INTERNAL_USE
#error "Please include mGui.h directly instead of mGui_Win32.h."
#else
#undef MGUI_WIN32_INTERNAL_USE 
#endif

#pragma once

#include "mGui.h"

#define WIN32_LEAN_AND_MEAN
#define WINVER 0x0500

#include <Windows.h>
#include <ShlObj.h>		// Dialogs
#include <CommDlg.h>    // Dialogs
#include <commctrl.h>
#include <Uxtheme.h>	// Removing styles
#include <ShellAPI.h>   // Drag and drop

#ifdef _MSC_VER
	#pragma comment(linker,"\"/manifestdependency:type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")
	#pragma comment(lib, "comctl32.lib")
	#pragma comment(lib, "uxtheme.lib")
	//#pragma comment(lib, "shell32.lib")
#endif


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// String types
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef MSTR_H_INCLUDED
std::string utf16to8(const wchar_t* sz);
std::wstring utf8to16(const char* sz);
#endif

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Class and struct definitions
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


struct MGControlData {
	HWND      ParentHWnd;
	HINSTANCE hInstance;
	HWND      hWnd;
	unsigned int ID;
};

class MGFontManager {
private:
	MGFontManager();
	static MGFontManager* singleton;

	HFONT hfMain; // Caches global MessageFont as an HFONT
	std::wstring fontFace;
	int  fontTwipSize;
	bool  bICCE; // Has InitCommonControlsEx() been called for this application?	

public:
	static MGFontManager* getInstance();

	int  LogheightToTwips( int lfHeight);
	int  LogheightToPixels(int lfHeight);
	void SetControlFont(HWND hWnd);
	void EnsureCommonControls();

	const wchar_t* getFontFace();
	int getFontTwipHeight();
	int getFontPixelHeight();
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// MGVirtualKey
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

unsigned int MGVKToNative(enum MGVirtualKeys mgvk);
enum MGVirtualKeys MGNativeToVK(unsigned int nativekey);
