/* mThread.cc
 * ``````````
 */

#include "mThread.h"

// MUTEXES
// ```````

#ifdef _WIN32

mtMutex::mtMutex() {	InitializeCriticalSection( &mtx );	}
mtMutex::~mtMutex() {	DeleteCriticalSection( &mtx );		}
void mtMutex::lock() {	EnterCriticalSection( &mtx );		}
void mtMutex::free() {	LeaveCriticalSection( &mtx );		}

#else

mtMutex::mtMutex() {	pthread_mutex_init( &mtx, 0 );		}
mtMutex::~mtMutex() {	pthread_mutex_destroy( &mtx );		}
void mtMutex::lock() {	pthread_mutex_lock( &mtx );			}
void mtMutex::free() {	pthread_mutex_unlock( &mtx );		}

#endif

// THREADS
// ```````

mtThread::mtThread() {
	thread_h = 0;
}

#ifdef _WIN32

mtThread::~mtThread() {
	if (thread_h) {
		CloseHandle(thread_h);
		thread_h = 0; // Not strictly necessary
	}
}

bool mtThread::start() {
	thread_h = CreateThread(
		0, 0, (LPTHREAD_START_ROUTINE)_entryPoint, this, 0, NULL
	);
	return (thread_h != 0);
}

unsigned long mtThread::join() {	
	DWORD exit_code;
	WaitForSingleObject(thread_h, INFINITE);
	GetExitCodeThread(thread_h, &exit_code);
	CloseHandle(thread_h);
	thread_h = 0;

	return exit_code;
}

void mtThread::abort() {
	TerminateThread(thread_h, -1);
	CloseHandle(thread_h);
	thread_h = 0;
}

bool mtThread::isRunning() {
	return (thread_h != 0);
}

MTHREAD_THREADENTRY_T mtThread::_entryPoint(void* data) {
	mtThread *mtt = (mtThread*)data;
	return mtt->threadMain();
}

#else

mtThread::~mtThread() {
	if (thread_h) pthread_cancel(thread_h);
}

bool mtThread::start() {
	return (pthread_create(&thread_h, NULL, _entryPoint, this) == 0);
}

unsigned long mtThread::join() {
	void* exit_code;
	pthread_join(thread_h, &exit_code);
	return (unsigned long)exit_code; // downcast
}

MTHREAD_THREADENTRY_T mtThread::_entryPoint(void* data) {
	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, 0);
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, 0);

	mtThread *mtt = (mtThread*)data;
	return (void*) mtt->threadMain(); // upcast
}

#endif

