/* mThread.h
 * `````````
 * Light threading wrapper.
 * Uses CRT/Win32 on Windows, and pthreads everywhere else.
 */

// HEADERS
// ```````

#ifdef _WIN32

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define MTHREAD_MUTEX_T CRITICAL_SECTION
#define MTHREAD_THREAD_T HANDLE
#define MTHREAD_THREADENTRY_T unsigned int WINAPI

#else

#include <pthread.h>

#define MTHREAD_MUTEX_T pthread_mutex_t
#define MTHREAD_THREAD_T pthread_t
#define MTHREAD_THREADENTRY_T void*

#endif

// MUTEXES
// ```````

class mtMutex {
public:
	mtMutex();
	~mtMutex();

	void lock();
	void free();

private:
	MTHREAD_MUTEX_T mtx;
};

// THREADS
// ```````

class mtThread {
public:
	mtThread();
	virtual ~mtThread();

	bool start();
	unsigned long join(); // Block until thread completes processing, get its return code
	void abort();
	bool isRunning(); // Not strictly whether it's running, just whether or not it's been join()/abort()/dtor'd yet

protected:
	virtual unsigned long threadMain() = 0;

private:
	static MTHREAD_THREADENTRY_T _entryPoint(void* data);
	MTHREAD_THREAD_T thread_h;
};
