// osal.cpp

#define _CRT_RAND_S
#include <stdlib.h>

#include "osal.h"

#include <Windows.h>

// For retrieving linktime
#ifdef _MSC_VER
	#pragma comment(lib, "imagehlp.lib")
#endif
#include <DbgHelp.h>
#include <ctime>

long getLinkTime() {
	HMODULE hModule = GetModuleHandle(0);
	return GetTimestampForLoadedLibrary(hModule);
};

std::string getLinkTimeString() {
	wchar_t timeBuff[64] = {0};

	time_t linktime = getLinkTime();	
	_wctime_s(timeBuff, 64, &linktime);
	
	std::wstring ret(timeBuff);
	return utf16to8(ret.c_str());
};

std::vector<std::string> filesInDirectory(const char* dir) {
	std::vector<std::string> ret;
	std::wstring mask = utf8to16(dir) + L"\\*";

	WIN32_FIND_DATA wfd = {0};
	HANDLE hFind = FindFirstFile(mask.c_str(), &wfd);
	if (hFind == INVALID_HANDLE_VALUE) return ret;

	do {
		if (wfd.dwFileAttributes & ~(FILE_ATTRIBUTE_DIRECTORY)) {
			ret.push_back(utf16to8(wfd.cFileName));
		}
	} while (FindNextFile(hFind, &wfd));

	FindClose(hFind);

	return ret;
};

void filesInDirectory_recurse(const char* dir, std::vector<std::string>& ret) {
	std::wstring mask = utf8to16(dir) + L"\\*";

	WIN32_FIND_DATA wfd = {0};
	HANDLE hFind = FindFirstFile(mask.c_str(), &wfd);
	if (hFind == INVALID_HANDLE_VALUE) return;

	do {
		if (wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			if (wfd.cFileName[0] != '.') {
				// Process directory
				// A little double-copying and double-encoding-conversion here..
				std::string subdir(dir);
				subdir.append("\\").append( utf16to8(wfd.cFileName));
				filesInDirectory_recurse(subdir.c_str(), ret);
			}
		} else {
			// File
			ret.push_back(std::string(dir).append("\\").append( utf16to8(wfd.cFileName) ));
		}
	} while (FindNextFile(hFind, &wfd));

	FindClose(hFind);
}

// Read entire file into std::string
bool file_get_contents(std::string filepath, std::string& result) {	
	FILE* f = 0;
	long flen;
		
	_wfopen_s(&f, utf8to16(filepath.c_str()).c_str(), L"rb");
	if (!f) return false;

		// Get length
		fseek(f, 0, SEEK_END);
		flen = ftell(f);
		fseek(f, 0, SEEK_SET);

		// Read into new buffer
		char* cBuff = new char[flen+1];
		memset(cBuff, 0, flen+1);
		fread(cBuff, 1, flen, f);
	fclose(f);

	result.assign(cBuff);
	delete[] cBuff;
	return true;
}

// Overwrite a file with the specified contents
bool file_put_contents(std::string filepath, std::string& data) {
	FILE* f = 0;
	_wfopen_s(&f, utf8to16(filepath.c_str()).c_str(), L"wb");
	if (!f) return false;

		fwrite(data.c_str(), 1, data.length(), f);
	fclose(f);

	return true;
};

std::string string_format(const char* fmt, ...) {
    if (!fmt) return std::string();
	
	va_list args;
    va_start(args, fmt);
	
    int   result = -1, length = 256;
    char *buffer = 0;

	while (result == -1) {
        if (buffer) delete [] buffer;
        buffer = new char[length + 1];
        memset(buffer, 0, length + 1);
        result = vsprintf_s(buffer, length, fmt, args);
        length *= 2;
    }

    std::string s(buffer);
    delete[] buffer;

    va_end(args);
    return s;
};

std::string utf16to8(const wchar_t* sz) {
    int size_needed = WideCharToMultiByte(CP_UTF8, 0, sz, -1, NULL, 0, NULL, NULL);
    std::string ret(size_needed, 0);
    WideCharToMultiByte(CP_UTF8, 0, sz, -1, &ret[0], size_needed, NULL, NULL);
	ret.resize(ret.size() - 1); // strip trailing null
    return ret;
};

std::wstring utf8to16(const char* sz) {
	int size_needed = MultiByteToWideChar(CP_UTF8, 0, sz, -1, NULL, 0);
    std::wstring ret(size_needed, 0);
    MultiByteToWideChar(CP_UTF8, 0, sz, -1, &ret[0], size_needed);
	ret.resize(ret.size() - 1); // strip trailing null
    return ret;
};

void strtolower(std::string& sz) {
	// FIXME: this skips any upper-utf8 characters
	if (! sz.length()) return;

	char *p = & sz[0], *e = p + sz.length();
	do {
		if (*p < 128) *p = tolower(*p);
	} while( *p ++ != *e );
};

void trim(std::string& sz) {
	if (! sz.length()) return;

	size_t first = sz.find_first_not_of(' '),
		last = sz.find_last_not_of(' ');

	if (first == 0) {
		if (last == sz.length()-1) {
			return; // nothing to do
		} else {
			sz.erase(last, std::string::npos);
		}
	} else {
		// full copy/switch
		sz.swap( sz.substr(first, last-first) );
	}
}

int explode(const char* needle, std::string& haystack, std::vector<std::string>& ret) {
	ret.clear();

	size_t needle_len = ::strlen(needle), pos = 0, tmp;
	while ((tmp = haystack.find(needle, pos)) != haystack.npos) {
		ret.push_back(haystack.substr(pos, tmp - pos));
		pos = tmp + needle_len;
	}
	ret.push_back(haystack.substr(pos));

	return ret.size();
}

// Write an INI structure to file
void ini_write(std::string filepath, ini_t& data) {
	std::string text;
	
	for (ini_t::iterator i = data.begin(), e = data.end(); i != e; i++) {
		text += "[" + i->first + "]\r\n";
		for (
			std::map<std::string,std::string>::iterator
			j = i->second.begin(), k = i->second.end(); j != k; j++
		) {
			text += j->first + "=" + j->second + "\r\n";
		}
		text += "\r\n";
	}

	file_put_contents(filepath, text);
}

// Parse an INI file structure
ini_t ini_read(std::string filepath) {
	ini_t ret;
	std::string sbuf;
	file_get_contents(filepath, sbuf);

	long fpo = 0, fpn = 0;
	std::string sCurSection = "";
	do {
		fpn = sbuf.find("\n", fpo+1);
		if (fpn != sbuf.npos) fpn++;
		std::string sLine = sbuf.substr(fpo, fpn-fpo);

		// Strip \r
		long rp; while ((rp = sLine.find("\r")) != sLine.npos) sLine.erase(rp, rp+1);
		
		// Process each line
		if (sLine.length() > 1 && sLine.at(0) != '#') {
			if (sLine.at(0) == '[') {
				// Section, find ]
				long ebp = sLine.find("]", 1);
				if (ebp != sLine.npos) {
					sCurSection = sLine.substr(1, ebp-1);
					ret[sCurSection];
				}

			} else {
				// Key/value, find =
				long eqp = sLine.find("=");
				if (eqp != sLine.npos) {
					ret[sCurSection][sLine.substr(0, eqp)] = sLine.substr(eqp+1);
				}
			}			
		}

		fpo = fpn;
	} while (fpo != sbuf.npos);

	return ret;
}

//

void exec_nowait(const char* binpath, const char* args) {
	STARTUPINFO sinf = {0};
	PROCESS_INFORMATION pinf = {0};
	sinf.cb = sizeof(STARTUPINFO);

	CreateProcess(
		utf8to16(binpath).c_str(),
		const_cast<wchar_t*>( utf8to16(args).c_str() ),
		NULL,
		NULL,
		FALSE,
		0,
		NULL,
		NULL,
		&sinf,
		&pinf
	);
}

unsigned int random_in_range(unsigned int min, unsigned int max) {

#ifdef WIN32
	// Platform-specific better PRNG
	unsigned int rv;
	rand_s(&rv);
	return (unsigned int)(
		(
			(double)rv / ( ((double)UINT_MAX) + 1.0 )
		)
		*
		(max-min + 1) + min
	);
#else

	// More uniform distribution
	return (unsigned int)(
		(
			(double)rand() / ( ((double)RAND_MAX) + 1.0  )
		)
		* (max-min + 1) + min
	);

#endif

	// Original
	//return (rand() % (max - min)) + min;
}